Attribute VB_Name = "mMaths"
Option Explicit

' Declare local variables ...

    Private P_dCalculatorMatrix As D3DMATRIX    ' Matrix used for calculations
 
Public Const PIFactor = 3.141592653 / 180
' MIDENTITY: Resets a Matrix to Identity
Public Sub MIdentity(dMatrix As D3DMATRIX)
    dMatrix.rc11 = 1
    dMatrix.rc12 = 0
    dMatrix.rc13 = 0
    dMatrix.rc14 = 0
    dMatrix.rc21 = 0
    dMatrix.rc22 = 1
    dMatrix.rc23 = 0
    dMatrix.rc24 = 0
    dMatrix.rc31 = 0
    dMatrix.rc32 = 0
    dMatrix.rc33 = 1
    dMatrix.rc34 = 0
    dMatrix.rc41 = 0
    dMatrix.rc42 = 0
    dMatrix.rc43 = 0
    dMatrix.rc44 = 1
End Sub

' MTRANSLATE: Translates a matrix along the axis
Public Function MTranslate(dSource As D3DMATRIX, ByVal nValueX As Single, ByVal nValueY As Single, ByVal nValueZ As Single) As D3DMATRIX

    ' Reset to identity matrix
    MIdentity P_dCalculatorMatrix
    
    ' Add calculations
    With P_dCalculatorMatrix
        .rc41 = nValueX
        .rc42 = nValueY
        .rc43 = nValueZ
    End With

    ' Apply transformations
    dSource = MMultiply(P_dCalculatorMatrix, dSource)

    ' Return result
    MTranslate = dSource
    
End Function

' MSCALE: Scales a matrix along the axis
Public Function MScale(dSource As D3DMATRIX, ByVal nValueX As Single, ByVal nValueY As Single, ByVal nValueZ As Single) As D3DMATRIX

    ' Reset to identity matrix
    MIdentity P_dCalculatorMatrix
    
    ' Add calculations
    With P_dCalculatorMatrix
        .rc11 = nValueX
        .rc22 = nValueY
        .rc33 = nValueZ
    End With
    
    ' Apply transformations
    dSource = MMultiply(P_dCalculatorMatrix, dSource)
    
    ' Return result
    MScale = dSource
    
End Function

' MROTATE: Rotates a matrix around the axis
Public Function MRotate(dSource As D3DMATRIX, ByVal nValueX As Single, ByVal nValueY As Single, ByVal nValueZ As Single) As D3DMATRIX

    ' Setup local variables ...
        
        Dim L_nCos As Single        ' Holds cosine value of angle
        Dim L_nSin As Single        ' Holds sine value of angle
        
    ' Do rotation around X-Axis ...
        If nValueX <> 0 Then
        
            ' Reset caluculator to identity matrix
            MIdentity P_dCalculatorMatrix
                
            ' Get angle values
            L_nCos = Cos(nValueX * PIFactor)
            L_nSin = Sin(nValueX * PIFactor)
            
            ' Add transformations
            With P_dCalculatorMatrix
                .rc22 = L_nCos
                .rc33 = L_nCos
                .rc23 = -L_nSin
                .rc32 = L_nSin
            End With
            
            ' Apply transformations
            dSource = MMultiply(P_dCalculatorMatrix, dSource)
            
        End If
    
    ' Do rotation around Y-Axis ...
        If nValueY <> 0 Then
        
            ' Reset caluculator to identity matrix
            MIdentity P_dCalculatorMatrix
            
            ' Get angle values
            L_nCos = Cos(nValueY * PIFactor)
            L_nSin = Sin(nValueY * PIFactor)
            
            ' Add transformations
            With P_dCalculatorMatrix
                .rc11 = L_nCos
                .rc33 = L_nCos
                .rc13 = L_nSin
                .rc31 = -L_nSin
            End With
            
            ' Apply transformations
            dSource = MMultiply(P_dCalculatorMatrix, dSource)
            
        End If
        
    ' Do rotation around Z-Axis ...
        If nValueZ <> 0 Then
        
            ' Reset caluculator to identity matrix
            MIdentity P_dCalculatorMatrix
            
            ' Get angle values
            L_nCos = Cos(nValueZ * PIFactor)
            L_nSin = Sin(nValueZ * PIFactor)
            
            ' Add transformations
            With P_dCalculatorMatrix
                .rc11 = L_nCos
                .rc22 = L_nCos
                .rc12 = -L_nSin
                .rc21 = L_nSin
            End With
            
            ' Apply transformations
            dSource = MMultiply(P_dCalculatorMatrix, dSource)
            
        End If
        
    ' Return result ...
        MRotate = dSource
    
End Function

' MMULTIPLY: Multiplies two matrices
Public Function MMultiply(dM1 As D3DMATRIX, dM2 As D3DMATRIX) As D3DMATRIX

    ' Calculate multiply ...
        With MMultiply
            
            .rc11 = dM1.rc11 * dM2.rc11 + dM1.rc21 * dM2.rc12 + dM1.rc31 * dM2.rc13 + dM1.rc41 * dM2.rc14
            .rc21 = dM1.rc11 * dM2.rc21 + dM1.rc21 * dM2.rc22 + dM1.rc31 * dM2.rc23 + dM1.rc41 * dM2.rc24
            .rc31 = dM1.rc11 * dM2.rc31 + dM1.rc21 * dM2.rc32 + dM1.rc31 * dM2.rc33 + dM1.rc41 * dM2.rc34
            .rc41 = dM1.rc11 * dM2.rc41 + dM1.rc21 * dM2.rc42 + dM1.rc31 * dM2.rc43 + dM1.rc41 * dM2.rc44
    
            .rc12 = dM1.rc12 * dM2.rc11 + dM1.rc22 * dM2.rc12 + dM1.rc32 * dM2.rc13 + dM1.rc42 * dM2.rc14
            .rc22 = dM1.rc12 * dM2.rc21 + dM1.rc22 * dM2.rc22 + dM1.rc32 * dM2.rc23 + dM1.rc42 * dM2.rc24
            .rc32 = dM1.rc12 * dM2.rc31 + dM1.rc22 * dM2.rc32 + dM1.rc32 * dM2.rc33 + dM1.rc42 * dM2.rc34
            .rc42 = dM1.rc12 * dM2.rc41 + dM1.rc22 * dM2.rc42 + dM1.rc32 * dM2.rc43 + dM1.rc42 * dM2.rc44
    
            .rc13 = dM1.rc13 * dM2.rc11 + dM1.rc23 * dM2.rc12 + dM1.rc33 * dM2.rc13 + dM1.rc43 * dM2.rc14
            .rc23 = dM1.rc13 * dM2.rc21 + dM1.rc23 * dM2.rc22 + dM1.rc33 * dM2.rc23 + dM1.rc43 * dM2.rc24
            .rc33 = dM1.rc13 * dM2.rc31 + dM1.rc23 * dM2.rc32 + dM1.rc33 * dM2.rc33 + dM1.rc43 * dM2.rc34
            .rc43 = dM1.rc13 * dM2.rc41 + dM1.rc23 * dM2.rc42 + dM1.rc33 * dM2.rc43 + dM1.rc43 * dM2.rc44
    
            .rc14 = dM1.rc14 * dM2.rc11 + dM1.rc24 * dM2.rc12 + dM1.rc34 * dM2.rc13 + dM1.rc44 * dM2.rc14
            .rc24 = dM1.rc14 * dM2.rc21 + dM1.rc24 * dM2.rc22 + dM1.rc34 * dM2.rc23 + dM1.rc44 * dM2.rc24
            .rc34 = dM1.rc14 * dM2.rc31 + dM1.rc24 * dM2.rc32 + dM1.rc34 * dM2.rc33 + dM1.rc44 * dM2.rc34
            .rc44 = dM1.rc14 * dM2.rc41 + dM1.rc24 * dM2.rc42 + dM1.rc34 * dM2.rc43 + dM1.rc44 * dM2.rc44
    
        End With
            
    
End Function

' MDEBUG: Print a given matrix in the debug window
Public Sub MDebug(dMatrix As D3DMATRIX)

    With dMatrix
        Debug.Print Format(.rc11, "0.00000") + " " + Format(.rc12, "0.00000") + " " + Format(.rc13, "0.00000") + " " + Format(.rc14, "0.00000")
        Debug.Print Format(.rc21, "0.00000") + " " + Format(.rc22, "0.00000") + " " + Format(.rc23, "0.00000") + " " + Format(.rc24, "0.00000")
        Debug.Print Format(.rc31, "0.00000") + " " + Format(.rc32, "0.00000") + " " + Format(.rc33, "0.00000") + " " + Format(.rc34, "0.00000")
        Debug.Print Format(.rc41, "0.00000") + " " + Format(.rc42, "0.00000") + " " + Format(.rc43, "0.00000") + " " + Format(.rc44, "0.00000")
    End With
    
End Sub

' MLOOKAT: Calculates a transformation matrix for the view
Public Function MLookAt(dCamPosition As D3DVECTOR, dCamLookAt As D3DVECTOR) As D3DMATRIX

    ' Setup local variables ...
    
        Dim L_dVU As D3DVECTOR
        Dim L_dVR As D3DVECTOR
        Dim L_dVView As D3DVECTOR
        Dim L_dVDefaultUp As D3DVECTOR
        
        ' Set world up vector (y-Axis is up)
        L_dVDefaultUp.y = -1
        
    ' Calculate camera transform ...
    
        ' Load result with identity matrix
        MIdentity MLookAt
        
        ' Calculate vector from position to look-at-point
        L_dVView = VNormalize(VSubtract(dCamLookAt, dCamPosition))
        
        ' Calculate right component of view vector
        L_dVR = VCrossProduct(L_dVDefaultUp, L_dVView)
        
        ' Calculate up component of view vector
        L_dVU = VCrossProduct(L_dVView, L_dVR)
        
        ' Normalize right and up
        L_dVR = VNormalize(L_dVR)
        L_dVU = VNormalize(L_dVU)
        
        ' Compose camera matrix
        With MLookAt
            .rc11 = L_dVR.x
            .rc21 = -L_dVR.y
            .rc31 = L_dVR.z
            .rc12 = L_dVU.x
            .rc22 = -L_dVU.y
            .rc32 = L_dVU.z
            .rc13 = L_dVView.x
            .rc23 = -L_dVView.y
            .rc33 = L_dVView.z
            .rc41 = -VDotProduct(L_dVR, dCamPosition)
            .rc42 = -VDotProduct(L_dVU, dCamPosition)
            .rc43 = -VDotProduct(L_dVView, dCamPosition)
        End With
    
End Function

' MPROJECT: Calculates a transformation matrix for the projection
Public Function MProject(nNear As Single, nFar As Single, nAngleFOV As Single) As D3DMATRIX

    ' Setup local variables ...
        Dim nC As Single
        Dim nS As Single
        Dim nQ As Single
        
    ' Calculate matrix ...
    
        nC = Cos(nAngleFOV * 0.5 * PIFactor)
        nS = Sin(nAngleFOV * 0.5 * PIFactor)
        nQ = nS / (1 - nNear / nFar)

        MIdentity MProject
        
        With MProject
            .rc11 = nC
            .rc22 = nC
            .rc33 = nQ
            .rc43 = -nQ * nNear
            .rc34 = nS
            .rc44 = 0
        End With

End Function

' VNORMALIZE: Returns normalized vector
Public Function VNormalize(dVector As D3DVECTOR) As D3DVECTOR
     DirectX.VectorNormalize dVector
  VNormalize = dVector
End Function


' VREFLECT: Calculates reflection of a vector upon a normal
Public Function VReflect(dRay As D3DVECTOR, dReflector As D3DVECTOR) As D3DVECTOR
    DirectX.VectorReflect VReflect, dRay, dReflector
End Function

' VCROSSPRODUCT: Calculates the cross product of two vectors
Public Function VCrossProduct(dV1 As D3DVECTOR, dV2 As D3DVECTOR) As D3DVECTOR
    DirectX.VectorCrossProduct VCrossProduct, dV1, dV2
End Function

' VDOTPRODUCT: Calculates the dot product of two vectors
Public Function VDotProduct(dV1 As D3DVECTOR, dV2 As D3DVECTOR) As Single
    VDotProduct = DirectX.VectorDotProduct(dV1, dV2)
End Function

' VSUBTRACT: Calculates the subtraction of V1 and V2
Public Function VSubtract(dV1 As D3DVECTOR, dV2 As D3DVECTOR) As D3DVECTOR
    VSubtract.x = dV1.x - dV2.x
    VSubtract.y = dV1.y - dV2.y
    VSubtract.z = dV1.z - dV2.z
End Function

' VADD: Calculates the addition of V1 and V2
Public Function VAdd(dV1 As D3DVECTOR, dV2 As D3DVECTOR) As D3DVECTOR
    VAdd.x = dV1.x + dV2.x
    VAdd.y = dV1.y + dV2.y
    VAdd.z = dV1.z + dV2.z
End Function

' VROTATE: Rotates a vector
Public Function VRotate(dV As D3DVECTOR, ByVal nValueX As Single, ByVal nValueY As Single, ByVal nValueZ As Single) As D3DVECTOR
    
    ' Setup local variables...
        Dim dVDir As D3DVECTOR  ' Holds axis for rotation
    
    ' Do rotations ...
        
        ' Rotate aroun X axis
        If nValueX <> 0 Then
            dVDir.x = 1
            dVDir.y = 0
            dVDir.z = 0
            DirectX.VectorRotate VRotate, dV, dVDir, nValueX * PIFactor
        End If
    
        ' Rotate aroun Y axis
        If nValueY <> 0 Then
            dVDir.x = 0
            dVDir.y = 1
            dVDir.z = 0
            DirectX.VectorRotate VRotate, dV, dVDir, nValueY * PIFactor
        End If
    
        ' Rotate aroun Z axis
        If nValueZ <> 0 Then
            dVDir.x = 0
            dVDir.y = 0
            dVDir.z = 1
            DirectX.VectorRotate VRotate, dV, dVDir, nValueZ * PIFactor
        End If
    
End Function



